<?php
/**
 * Created by PhpStorm.
 * User: Remi
 * Date: 29/03/2019
 * Time: 23:30
 */

namespace XenConcept\LoginAsUser\Entity;

use XF\Mvc\Entity\Entity;
use XF\Mvc\Entity\Structure;

/**
 * COLUMNS
 * @property int|null log_id
 * @property int from_user_id
 * @property string from_username
 * @property int to_user_id
 * @property string to_username
 * @property int log_date
 * @property string reason
 * @property string ip_address
 *
 * GETTERS
 * No getters
 *
 * RELATIONS
 * @property \XF\Entity\User FromUser
 * @property \XF\Entity\User ToUser
 */
class Log extends Entity
{

    public static function getStructure(Structure $structure)
    {
        $structure->table      = 'xf_xc_lau_logs';
        $structure->shortName  = 'XenConcept\LoginAsUser:Log';
        $structure->primaryKey = 'log_id';
        $structure->columns = [
            'log_id' => ['type' => self::UINT, 'autoIncrement' => true, 'nullable' => true],
            'from_user_id' => ['type' => self::UINT, 'default' => 0],
            'from_username' => ['type' => self::STR, 'maxLength' => 50,
                'required' => 'please_enter_valid_name'
            ],
            'to_user_id' => ['type' => self::UINT, 'default' => 0],
            'to_username' => ['type' => self::STR, 'maxLength' => 50,
                'required' => 'please_enter_valid_name'
            ],
            'log_date' => ['type' => self::UINT, 'default' => \XF::$time],
            'reason' => ['type' => self::STR, 'default' => ''],
            'ip_address' => ['type' => self::BINARY, 'maxLength' => 16],
        ];
        $structure->getters = [];
        $structure->relations = [
            'FromUser' => [
                'entity'     => 'XF:User',
                'type'       => self::TO_ONE,
                'conditions' => [['user_id', '=', '$from_user_id']],
                'primary'    => true
            ],
            'ToUser' => [
                'entity'     => 'XF:User',
                'type'       => self::TO_ONE,
                'conditions' => [['user_id', '=', '$to_user_id']],
                'primary'    => true
            ]
        ];

        $structure->defaultWith = ['FromUser', 'ToUser'];

        return $structure;
    }

}