<?php
/*************************************************************************
 * Login As User - XenConcept (c) 2019
 * All Rights Reserved.
 **************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at Try it like it buy it :)
 *************************************************************************/

namespace XenConcept\LoginAsUser\Service\LoginAsUser;

use XF\Service\AbstractService;

class Conversation extends AbstractService
{
    /**
     * @var \XF\Entity\User
     */
    protected $fromUser;

    /**
     * @var \XF\Entity\User
     */
    protected $toUser;

    protected $reason;

    protected $title;
    protected $message;
    protected $options;

    public function __construct(\XF\App $app, \XF\Entity\User $toUser, \XF\Entity\User $fromUser, $reason)
    {
        parent::__construct($app);

        $this->toUser   = $toUser;
        $this->fromUser = $fromUser;
        $this->reason   = $reason;
    }

    public function setTitle($title)
    {
        $this->title = $title;
    }

    public function setBody($message)
    {
        $this->message = $message;
    }

    public function setOptions($options)
    {
        $this->options = $options;
    }

    public function send()
    {
        /** @var \XF\Service\Conversation\Creator $creator */
        $creator = $this->service('XF:Conversation\Creator', $this->fromUser);


        $language = $this->app->language($this->toUser->language_id);
        $title    = $this->replacePhrases($this->title, $language);
        $message  = $this->replacePhrases($this->getPreparedMessage($this->message), $language);

        $optionsInput = [
            'open_invite'       => $this->options['open_invite'],
            'conversation_open' => $this->options['conversation_open']
        ];

        $creator->setRecipients($this->toUser->username);
        $creator->setIsAutomated();
        $creator->setContent($title, $message);
        $creator->setOptions($optionsInput);
        $creator->save();

        return;
    }

    protected function getPreparedMessage($message)
    {
        $message = str_replace('{username}', $this->toUser->username, $message);
        $message = str_replace('{reason}', $this->reason, $message);
        $message = str_replace(PHP_EOL, '<br>', $message);
        $bbCode = \XF\Html\Renderer\BbCode::renderFromHtml($message);

        return \XF::cleanString($bbCode);
    }

    protected function replacePhrases($string, \XF\Language $language)
    {
        return $this->app->stringFormatter()->replacePhrasePlaceholders($string, $language);
    }

}