<?php

/*************************************************************************
 * Login As User - XenConcept (c) 2019
 * All Rights Reserved.
 **************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at Try it like it buy it :)
 *************************************************************************/

namespace XenConcept\LoginAsUser\Service\LoginAsUser;

use XF\Service\AbstractService;

class Login extends AbstractService
{

    /**
     * @var \XF\Entity\User
     */
    protected $fromUser;

    /**
     * @var \XF\Entity\User
     */
    protected $toUser;

    protected $toUsername;
    protected $reason;

    protected $errors = [];

    public function __construct(\XF\App $app, \XF\Entity\User $fromUser)
    {
        parent::__construct($app);
        $this->fromUser = $fromUser;
    }

    public function getErrors()
    {
        return $this->errors;
    }

    public function setUsername($username)
    {
        $this->toUsername = $username;
    }

    public function setReason($reason)
    {
        $this->reason = $reason;
    }

    public function preValidateLogin()
    {
        $toUsername = $this->toUsername;
        $reason = $this->reason;

        if (empty($toUsername))
        {
            $this->errors[] = \XF::phrase('xc_lau_please_select_user');
        }

        $usernameExplode = explode(',', $toUsername);
        if (count($usernameExplode) != 1)
        {
            $this->errors[] = \XF::phrase('xc_lau_you_can_not_select_multiple_users');
            return false;
        }

        $this->toUser = $this->getUserRepo()->getUserByNameOrEmail($toUsername);
        $toUser = $this->toUser;

        if(empty($toUser) || !$toUser instanceof \XF\Entity\User)
        {
            $this->errors[] = \XF::phrase('requested_user_not_found');
            return false;
        }

        if($toUser->user_id == \XF::visitor()->user_id)
        {
            $this->errors[] = \XF::phrase('you_already_logged_in');
            return false;
        }

        $disallowedIDs = preg_split('/\s+/', trim(\XF::options()->xc_login_as_user_disallowed_user_ids), -1, PREG_SPLIT_NO_EMPTY);

        if (in_array($toUser->user_id, $disallowedIDs))
        {
            $this->errors[] = \XF::phrase('xc_lau_it_is_not_allowed_to_log_in_with_x', ['username' => $toUser->username]);
            return false;
        }

        if (empty($reason) && \XF::options()->xc_login_as_user_require_reason)
        {
            $this->errors[] = \XF::phrase('xc_lau_you_must_enter_a_reason');
            return false;
        }

        return true;
    }

    public function sendNotification($messageTitle, $messageBody, $options)
    {
        /** @var \XenConcept\LoginAsUser\Service\LoginAsUser\Conversation $conversationService */
        $conversationService = $this->service('XenConcept\LoginAsUser:LoginAsUser\Conversation', $this->toUser, $this->fromUser, $this->reason);
        $conversationService->setTitle($messageTitle);
        $conversationService->setBody($messageBody);
        $conversationService->setOptions($options);
        $conversationService->send();
    }

    public function processLogin()
    {
        $fromUser = $this->fromUser;
        $toUser = $this->toUser;

        /** @var \XenConcept\LoginAsUser\Service\LoginAsUser\Log $logService */
        $logService = $this->service('XenConcept\LoginAsUser:LoginAsUser\Log', $toUser, $fromUser, $this->reason);
        $logService->saveLog();

        return $toUser;
    }

    /**
     * @return \XF\Repository\User
     */
    protected function getUserRepo()
    {
        return $this->repository('XF:User');
    }

}