<?php

/*************************************************************************
 * Login As User - XenConcept (c) 2019
 * All Rights Reserved.
 **************************************************************************
 * This file is subject to the terms and conditions defined in the Licence
 * Agreement available at Try it like it buy it :)
 *************************************************************************/

namespace XenConcept\LoginAsUser\XF\Pub\Controller;

class Misc extends XFCP_Misc
{

    protected function setupLoginAsUser(\XF\Entity\User $fromUser)
    {
        /** @var \XenConcept\LoginAsUser\Service\LoginAsUser\Login $loginService */
        $loginService = $this->service('XenConcept\LoginAsUser:LoginAsUser\Login', $fromUser);
        /** @var \XF\ControllerPlugin\Editor $editorPlugin */
        $editorPlugin = $this->plugin('XF:Editor');

        $loginService->setUsername($this->filter('username', 'str'));
        $loginService->setReason($editorPlugin->fromInput('reason'));

        if (!$loginService->preValidateLogin())
        {
            Throw $this->exception($this->error($loginService->getErrors()));
        }

        $notification    = $this->filter('notification', 'array');
        $notificationAcp = \XF::options()->xc_login_as_user_send_conversation;

        if (isset($notification['sendNotification']) && intval($notification['sendNotification']))
        {
            $messageBody = $editorPlugin->convertToBbCode($notification['messageBody_html']);
            $loginService->sendNotification($notification['messageTitle'], $messageBody, [
                'open_invite' => isset($notification['messageOpenInvite']) ? $notification['messageOpenInvite'] : false,
                'conversation_open' => isset($notification['conversation_open']) ? false : true,
            ]);
        }
        elseif ($notificationAcp['messageEnabled'])
        {
            $messageBody = $editorPlugin->convertToBbCode($notificationAcp['messageBody']);
            $loginService->sendNotification($notificationAcp['messageTitle'], $messageBody, [
                'open_invite' => $notificationAcp['messageOpenInvite'],
                'conversation_open' => $notificationAcp['messageLocked']
            ]);
        }

        $toUser   = $loginService->processLogin();

        $formRedirect   = $this->filter('revert_to', 'str');
        $optionRedirect = \XF::options()->xc_login_as_user_url_to_revert;

        if (empty($formRedirect) && empty($optionRedirect))
        {
            $urlRevert = '';
        }

        if (!empty($formRedirect))
        {
            $urlRevert = $formRedirect;
        }
        elseif (!empty($optionRedirect))
        {
            $urlRevert = $optionRedirect;
        }
        else
        {
            $urlRevert = '';
        }

        /** @var \XenConcept\LoginAsUser\XF\ControllerPlugin\Login $loginPlugin */
        $loginPlugin = $this->plugin('XF:Login');
        $loginPlugin->loginAsUser($fromUser, $toUser, $urlRevert);

        return true;
    }

    /**
     * @return \XF\Mvc\Reply\Redirect|\XF\Mvc\Reply\View
     * @throws \XF\Mvc\Reply\Exception
     */
    public function actionLoginAsUser()
    {
        /** @var \XenConcept\LoginAsUser\XF\Entity\User $visitor */
        $visitor  = \XF::visitor();
        if (!$visitor->canUseLoginAsUser())
        {
            return $this->noPermission();
        }

        $user = null;

        if ($user = $this->filter('user_id', 'int'))
        {
            $user = $this->em()->find('XF:User', $user);
        }

        if ($this->isPost())
        {

            $this->setupLoginAsUser(\XF::visitor());

            $formRedirect   = $this->filter('redirect_to', 'str');
            $optionRedirect = \XF::options()->xc_login_as_user_url_to_redirect;

            if (empty($formRedirect) && empty($optionRedirect))
            {
                return $this->redirect($this->buildLink('index'));
            }

            if (!empty($formRedirect))
            {
                return $this->redirect($formRedirect);
            }
            elseif (!empty($optionRedirect))
            {
                return $this->redirect($optionRedirect);
            }
            else
            {
                return $this->redirect($this->buildLink('index'));
            }
        }
        else
        {
            $defaultRedirect = $this->getDynamicRedirect();

            $viewParams = [
                'user' => $user,

                'defaultRedirect' => $defaultRedirect,

                'username' => $user ? $user->username : ''
            ];

            return $this->view('XenConcept\LoginAsUser:Misc\LoginAsUser', 'xc_lau_misc_login_as_user', $viewParams);
        }
    }

    public function actionLogoutAsUser()
    {
        $session = $this->session();
        $sessionLau = $session->get('login_as_user');

        if (empty($sessionLau))
        {
            return $this->notFound();
        }

        $urlRevert = $sessionLau['url_revert'];
        $fromUser = \XF::visitor();
        /** @var \XF\Entity\User $toUser */
        $toUser   = $this->em()->find('XF:User', $sessionLau['from_user_id']);

        /** @var \XenConcept\LoginAsUser\Service\LoginAsUser\Log $logService */
        $logService = $this->service('XenConcept\LoginAsUser:LoginAsUser\Log', $toUser, $fromUser, '', 'logout');
        $logService->saveLog();

        /** @var \XenConcept\LoginAsUser\XF\ControllerPlugin\Login $loginPlugin **/
        $loginPlugin = $this->plugin('XF:Login');
        $loginPlugin->logoutAsUser($fromUser, $toUser);


        if (empty($urlRevert))
        {
            return $this->redirect($this->getDynamicRedirect());
        }
        else
        {
            return $this->redirect($urlRevert);
        }
    }


    /**
     * @return \XF\Repository\User
     */
    protected function getUserRepo()
    {
        return $this->repository('XF:User');
    }
}